/****************************************************************************
*
*               Laborator SMP - 80386EX Z3/EV Real Mode
*
*    Portions Copyright (c) 2010 Mihai Matei. All Rights Reserved.
*
*  ========================================================================
*
*    This file contains Original Code and/or Modifications of Original
*    Code as defined in and that are subject to the Apache License,
*    Version 2.0 (the "License"). You may not use this file
*    except in compliance with the License. BY USING THIS FILE YOU AGREE TO
*    ALL TERMS AND CONDITIONS OF THE LICENSE. A copy of the License is
*    provided with the Original Code and Modifications, and is also
*    available at http://www.apache.org/licenses/LICENSE-2.0
*
*    The Original Code and all software distributed under the License are
*    distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
*    EXPRESS OR IMPLIED, AND SYBASE AND ALL CONTRIBUTORS HEREBY DISCLAIM
*    ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR
*    NON-INFRINGEMENT. Please see the License for the specific language
*    governing rights and limitations under the License.
*
*  ========================================================================
*
* Description:  Entry point for SMP Z3/EV Project
*
****************************************************************************/

#include "platform_io.h"

static SerialConfig serialConf;

// remove the below to work with serial port on the EV/Z3 board
#define __TEST

static int  dbgSerialWrite(SerialPort com, const unsigned char *buffer, const int size)
{
#ifndef __TEST
    return kIo.SerialWrite(COM1, buffer, size);
#else
    unsigned char aux[4];
    aux[0] = buffer[0]; aux[1] = buffer[1]; aux[2] = buffer[2]; aux[3] = buffer[3];
    // put a breakpoint here to debug
    return 0;
#endif
}

char* map   = "xgxxBxxBxRBRRGRRRR";
int norows  = 6;
int nocols  = 3;
int nototal = 18;
unsigned char robotPos[2] = {0, 1}; // row, col
const char robotLetter = 'Z';

static void print_map()
{
    char buffer[4], row, col, color;
    int i;
    row = col = color = 0;

    // use a for to simulate the LOOP iteration
    for (i = 0; i < nototal; i++) {
        __asm {
            ; the rest is copied from the asm robozzle template
            PUSHA
            mov SI, i
            mov BX, map
            ADD BX, SI
            MOV AH, byte ptr [BX]
            MOV color, AH

            ; check to see if a new row was added
            MOV AX, SI
            MOV BX, nocols
            DIV BL
            MOV DH, AL ; AL = current row = AX / BL
            MOV DL, AH ; AH = current col = AX % BL
            ; set the registers in the row col C variables
            mov row, DH
            mov col, DL
            POPA
        }
        // now we can send the current char
        buffer[0] = color; buffer[1] = row; buffer[2] = col;
        dbgSerialWrite(COM1, buffer, 3);
    }

    // now print robot position
    buffer[0] = robotLetter; buffer[1] = robotPos[0]; buffer[2] = robotPos[1]; buffer[3] = map[robotPos[0] * nocols + robotPos[1]];
    dbgSerialWrite(COM1, buffer, 4);
}

int main()
{
    int key = 0;

    /* display our welcome message */
    kIo.DisplayString("Robozzle Demo   ");
    
    /* Serial Initialization */
    serialConf.baudRate   = b9600;
    serialConf.parity     = none;
    serialConf.stopBits   = one;
    serialConf.wordLength = eight;
    /* In order to check what EV/Z3 board is echoing on the serial port
     * open Hyperterminal, create a new connection using the above parameters,
     * Flow control: Hardware and Settings/Emulation: ANSI, connect and type some chars in it */
    
    kIo.SerialInit(&serialConf);

    print_map();

    /* wait for a key to be pressed */
    key = kIo.ReadKey();

    return 0;
}

